@@ -11,6 +11,11 @@
 #' @param func Function object to call in the background R process.
 #'   Please read the notes for the similar argument of [r()].
 #' @param args Arguments to pass to the function. Must be a list.
+#' @param package Whether to keep the environment of `func` when passing
+#'   it to the other package. Possible values are:
+#'   * `FALSE`: reset the environment to `.GlobalEnv`. This is the default.
+#'   * `TRUE`: keep the environment as is.
+#'   * `pkg`: set the environment to the `pkg` package namespace.
 #'
 #' @examplesIf FALSE
 #' rs <- r_ression$new()
@@ -31,6 +36,18 @@ r_session <- R6::R6Class(
 
   public = list(
 
+    #' @field status
+    #' Status codes returned by `read()`.
+    status = list(
+      DONE        = 200L,
+      STARTED     = 201L,
+      ATTACH_DONE = 202L,
+      MSG         = 301L,
+      EXITED      = 500L,
+      CRASHED     = 501L,
+      CLOSED      = 502L
+    ),
+
     #' @description
     #' creates a new R background process. It can wait for the process to
     #' start up (`wait = TRUE`), or return immediately, i.e. before
@@ -52,8 +69,8 @@ r_session <- R6::R6Class(
     #' R session. It throws an error if the function call generated an
     #' error in the child process.
     #' @return The return value of the R expression.
-    run = function(func, args = list())
-      rs_run(self, private, func, args),
+    run = function(func, args = list(), package = FALSE)
+      rs_run(self, private, func, args, package),
 
     #' @description
     #' Similar to `$run()`, but returns the standard output and error of
@@ -70,15 +87,15 @@ r_session <- R6::R6Class(
     #'   error thrown in the subprocess. Otherwise it is `NULL`.
     #' * `code`, `message`: These fields are used by call internally and
     #'   you can ignore them.
-    run_with_output = function(func, args = list())
-      rs_run_with_output(self, private, func, args),
+    run_with_output = function(func, args = list(), package = FALSE)
+      rs_run_with_output(self, private, func, args, package),
 
     #' @description
     #' Starts running a function in the background R session, and
     #' returns immediately. To check if the function is done, call the
     #' `poll_process()` method.
-    call = function(func, args = list())
-      rs_call(self, private, func, args),
+    call = function(func, args = list(), package = FALSE)
+      rs_call(self, private, func, args, package),
 
     #' @description
     #' Poll the R session with a timeout. If the session has finished the
@@ -120,19 +137,23 @@ r_session <- R6::R6Class(
     #'
     #' @return `NULL` if no events are available. Otherwise a named list,
     #'   which is also a `callr_session_result` object. The list always has
-    #'   a `code` entry which is the type of the event. Event types:
-    #'   * `200`: The computation is done, and the event includes the
-    #'      result, in the same form as for the `run()` method.
-    #'   * `201`: An R session that was in 'starting' state is ready to go.
-    #'   * `202`: Used by the `attach()` method.
-    #'   * `301`: A message from the subprocess. The message is a condition
-    #'      object with class `callr_message`. (It typically has other
-    #'      classes, e.g. `cli_message` for output from the cli package.
-    #'   * `500`: The R session finished cleanly. This means that the
-    #'      evaluated expression quit R.
-    #'   * `501`: The R session crashed or was killed.
-    #'   * `502`: The R session closed its end of the connection that
-    #'      callr uses for communication.
+    #'   a `code` entry which is the type of the event. See also
+    #'   `r_session$public_fields$status` for symbolic names of the
+    #'   event types.
+    #'   * `200`: (`DONE`) The computation is done, and the event includes
+    #'      the result, in the same form as for the `run()` method.
+    #'   * `201`: (`STARTED`) An R session that was in 'starting' state is
+    #'      ready to go.
+    #'   * `202`: (`ATTACH_DONE`) Used by the `attach()` method.
+    #'   * `301`: (`MSG`) A message from the subprocess. The message is a
+    #'      condition object with class `callr_message`. (It typically has
+    #'      other classes, e.g. `cli_message` for output from the cli
+    #'      package.
+    #'   * `500`: (`EXITED`) The R session finished cleanly. This means
+    #'      that the evaluated expression quit R.
+    #'   * `501`: (`CRASHED`) The R session crashed or was killed.
+    #'   * `502`: (`CLOSED`) The R session closed its end of the connection
+    #'      that callr uses for communication.
 
     read = function()
       rs_read(self, private),
@@ -206,8 +227,14 @@ r_session <- R6::R6Class(
     func_file = NULL,
     res_file = NULL,
 
-    get_result_and_output = function()
-      rs__get_result_and_output(self, private),
+    buffer = NULL,
+    read_buffer = function()
+      rs__read_buffer(self, private),
+    read_message = function()
+      rs__read_message(self, private),
+
+    get_result_and_output = function(std = FALSE)
+      rs__get_result_and_output(self, private, std),
     report_back = function(code, text = "")
       rs__report_back(self, private, code, text),
     write_for_sure = function(text)
@@ -261,7 +288,18 @@ rs_init <- function(self, private, super, options, wait, wait_timeout) {
     }
 
     if (pr["process"] == "ready") {
-      self$read()
+      msg <- self$read()
+      out <- paste0(out, msg$stdout)
+      err <- paste0(err, msg$stderr)
+      if (msg$code != 201) {
+        data <- list(
+          status = self$get_exit_status(),
+          stdout = out,
+          stderr = err,
+          timeout = FALSE
+        )
+        throw(new_callr_error(data, "Failed to start R session"))
+      }
     } else if (pr["process"] != "ready") {
       cat("stdout:]\n", out, "\n")
       cat("stderr:]\n", err, "\n")
@@ -273,21 +311,98 @@ rs_init <- function(self, private, super, options, wait, wait_timeout) {
 }
 
 rs_read <- function(self, private) {
-  out <- processx::processx_conn_read_lines(private$pipe, 1)
+  if (!is.null(private$buffer)) {
+    # There is a partial message in the buffer, try to finish it.
+    out <- private$read_buffer()
+  } else {
+    # A new message.
+    out <- private$read_message()
+  }
   if (!length(out)) {
     if (processx::processx_conn_is_incomplete(private$pipe)) return()
     if (self$is_alive()) {
-      self$kill()
-      out <- "502 R session closed the process connection, killed"
+      # We do this in on.exit(), because parse_msg still reads the streams
+      on.exit(self$kill(), add = TRUE)
+      out <- list(header = list(
+        code = 502, length = 0,
+        rest = "R session closed the process connection, killed"
+      ))
     } else if (identical(es <- self$get_exit_status(), 0L)) {
-      out <- "500 R session finished cleanly"
+      out <- list(header = list(
+        code = 500, length = 0,
+        rest = "R session finished cleanly"
+      ))
     } else {
-      out <- paste0("501 R session crashed with exit code ", es)
+      out <- list(header = list(
+        code = 501, length = 0,
+        rest = paste0("R session crashed with exit code ", es)
+      ))
     }
   }
   if (length(out)) private$parse_msg(out)
 }
 
+rs__read_buffer <- function(self, private) {
+  # There is a partial message in the buffer already, we need to
+  # read some more
+  need <- private$buffer$header$length - private$buffer$got
+  chunk <- processx::processx_conn_read_chars(private$pipe, need)
+  got <- nchar(chunk)
+  if (got == 0) {
+    # make this special case fast
+    NULL
+  } else if (got == need) {
+    msg <- list(
+      header = private$buffer$header,
+      body = paste(c(private$buffer$chunks, list(chunk)), collapse = "")
+    )
+    private$buffer <- NULL
+    msg
+  } else {
+    private$buffer$got <- private$buffer$got + got
+    private$buffer$chunks <- c(private$buffer$chunks, list(chunk))
+    NULL
+  }
+}
+
+rs__read_message <- function(self, private) {
+  # A new message, we can surely read the first line
+  out <- processx::processx_conn_read_lines(private$pipe, 1)
+  if (length(out) == 0) return(NULL)
+
+  header <- rs__parse_header(out)
+  body <- ""
+  if (header$length > 0) {
+    body <- processx::processx_conn_read_chars(
+      private$pipe,
+      header$length
+    )
+  }
+  got <- nchar(body)
+  if (got < header$length) {
+    # Partial message
+    private$buffer <- list(
+      header = header,
+      got = got,
+      chunks = list(body)
+    )
+    NULL
+  } else {
+    list(header = header, body = body)
+  }
+}
+
+rs__parse_header <- function(line) {
+  parts <- strsplit(line, " ", fixed = TRUE)[[1]]
+  parts2 <- suppressWarnings(as.integer(parts[1:2]))
+  rest <- paste(parts[-(1:2)], collapse = " ")
+  header <- list(code = parts2[1], length = parts2[2], rest = rest)
+  if (is.na(header$code) || is.na(header$length)) {
+    stop("Internal callr error, invalid message header")
+  }
+  header
+}
+
 rs_close <- function(self, private, grace) {
   processx::processx_conn_close(self$get_input_connection())
   self$poll_process(grace)
@@ -301,7 +416,7 @@ rs_close <- function(self, private, grace) {
   processx::processx_conn_close(self$get_error_connection())
 }
 
-rs_call <- function(self, private, func, args) {
+rs_call <- function(self, private, func, args, package) {
 
   ## We only allow a new command if the R session is idle.
   ## This allows keeping a clean state
@@ -313,18 +428,19 @@ rs_call <- function(self, private, func, args) {
   ## Save the function in a file
   private$options$func <- func
   private$options$args <- args
+  private$options$package <- package
   private$options$func_file <- save_function_to_temp(private$options)
-  private$options$result_file <- tempfile()
+  private$options$result_file <- tempfile("callr-rs-result-")
   private$options$tmp_files <-
     c(private$options$tmp_files, private$options$func_file,
       private$options$result_file)
 
   ## Maybe we need to redirect stdout / stderr
   re_stdout <- if (is.null(private$options$stdout)) {
-    private$tmp_output_file <- tempfile()
+    private$tmp_output_file <- tempfile("callr-rs-stdout-")
   }
   re_stderr <- if (is.null(private$options$stderr)) {
-    private$tmp_error_file <- tempfile()
+    private$tmp_error_file <- tempfile("callr-rs-stderr-")
   }
 
   pre <- rs__prehook(re_stdout, re_stderr)
@@ -349,8 +465,8 @@ rs_call <- function(self, private, func, args) {
   private$state <- "busy"
 }
 
-rs_run_with_output <- function(self, private, func, args) {
-  self$call(func, args)
+rs_run_with_output <- function(self, private, func, args, package) {
+  self$call(func, args, package)
 
   go <- TRUE
   res <- NULL
@@ -392,8 +508,8 @@ rs_run_with_output <- function(self, private, func, args) {
   res
 }
 
-rs_run <- function(self, private, func, args) {
-  res <- rs_run_with_output(self, private, func, args)
+rs_run <- function(self, private, func, args, package) {
+  res <- rs_run_with_output(self, private, func, args, package)
   if (is.null(res$error)) {
     res$result
   } else{
@@ -537,7 +653,10 @@ rs__attach_wait <- function(self, private) {
 }
 
 rs__report_back <- function(self, private, code, text) {
-  cmd <- paste0(deparse(rs__status_expr(code, text, fd = 3)), "\n")
+  cmd <- paste0(
+    deparse(rs__status_expr(code, text, fd = 3)),
+    "\n"
+  )
   private$write_for_sure(cmd)
 }
 
@@ -550,19 +669,20 @@ rs__write_for_sure <- function(self, private, text) {
 }
 
 rs__parse_msg <- function(self, private, msg) {
-  s <- strsplit(msg, " ", fixed = TRUE)[[1]]
-  code <- as.integer(s[1])
-  message <- paste(s[-1], collapse = " ")
-  if (substr(message, 1, 8) == "base64::") {
+  code <- as.character(msg$header$code)
+  message <- msg$body
+  if (length(message) && substr(message, 1, 8) == "base64::") {
     message <- substr(message, 9, nchar(message))
     message <- unserialize(processx::base64_decode(message))
+  } else {
+    message <- msg$header$rest
   }
 
-  if (! s[1] %in% names(rs__parse_msg_funcs)) {
-    throw(new_error("Unknown message code: `", s[1], "`"))
+  if (! code %in% names(rs__parse_msg_funcs)) {
+    throw(new_error("Unknown message code: `", code, "`"))
   }
   structure(
-    rs__parse_msg_funcs[[ s[1] ]](self, private, code, message),
+    rs__parse_msg_funcs[[code]](self, private, msg$header$code, message),
     class = "callr_session_result")
 }
 
@@ -597,7 +717,7 @@ rs__parse_msg_funcs[["301"]] <- function(self, private, code, message) {
 
 rs__parse_msg_funcs[["500"]] <- function(self, private, code, message) {
   private$state <- "finished"
-  res <- private$get_result_and_output()
+  res <- private$get_result_and_output(std = TRUE)
   c(list(code = code, message = message), res)
 }
 
@@ -606,7 +726,7 @@ rs__parse_msg_funcs[["501"]] <- function(self, private, code, message) {
   err <- structure(
     list(message = message),
     class = c("error", "condition"))
-  res <- private$get_result_and_output()
+  res <- private$get_result_and_output(std = TRUE)
   res$error <- err
   c(list(code = code, message = message), res)
 }
@@ -618,7 +738,7 @@ rs__status_expr <- function(code, text = "", fd = 3L) {
     local({
       pxlib <- as.environment("tools:callr")$`__callr_data__`$pxlib
       code_ <- code; fd_ <- fd; text_ <- text
-      data <- paste0(code_, " ", text_, "\n")
+      data <- paste0(code_, " 0 ", text_, "\n")
       pxlib$write_fd(as.integer(fd), data)
     }),
     list(code = code, fd = fd, text = text)
@@ -659,18 +779,22 @@ rs__posthook <- function(stdout, stderr) {
   substitute({ o; e }, list(o = oexpr, e = eexpr))
 }
 
-rs__get_result_and_output <- function(self, private) {
+rs__get_result_and_output <- function(self, private, std) {
 
   ## Get stdout and stderr
   stdout <- if (!is.null(private$tmp_output_file) &&
              file.exists(private$tmp_output_file)) {
     tryCatch(suppressWarnings(read_all(private$tmp_output_file)),
              error = function(e) "")
+  } else if (std && self$has_output_connection()) {
+    tryCatch(self$read_all_output(), error = function(err) NULL)
   }
   stderr <- if (!is.null(private$tmp_error_file) &&
              file.exists(private$tmp_error_file)) {
     tryCatch(suppressWarnings(read_all(private$tmp_error_file)),
              error = function(e) "")
+  } else if (std && self$has_error_connection()) {
+    tryCatch(self$read_all_error(), error = function(err) NULL)
   }
   unlink(c(private$tmp_output_file, private$tmp_error_file))
   private$tmp_output_file <- private$tmp_error_file <- NULL
@@ -740,7 +864,12 @@ r_session_options_default <- function() {
     stdout = NULL,
     stderr = NULL,
     error = getOption("callr.error", "error"),
-    cmdargs = c("--no-readline", "--slave", "--no-save", "--no-restore"),
+    cmdargs = c(
+      if (os_platform() != "windows") "--no-readline",
+      "--slave",
+      "--no-save",
+      "--no-restore"
+    ),
     system_profile = FALSE,
     user_profile = "project",
     env = c(TERM = "dumb"),
