@@ -14,12 +14,30 @@
 
 default_repos <- function() {
   opt <- getOption("repos")
+  was_list <- is.list(opt)
   if (! "CRAN" %in% names(opt) || opt[["CRAN"]] == "@CRAN@") {
     opt[["CRAN"]] <- "https://cloud.r-project.org"
   }
+  if (!was_list) opt <- unlist(opt)
   opt
 }
 
+remove_source <- function(x) {
+  if (is.function(x)) {
+    body(x) <- remove_source(body(x))
+    x
+  } else if (is.call(x)) {
+    attr(x, "srcref") <- NULL
+    attr(x, "wholeSrcref") <- NULL
+    attr(x, "srcfile") <- NULL
+
+    x[] <- lapply(x, remove_source)
+    x
+  } else {
+    x
+  }
+}
+
 `%||%` <- function(l, r) if (is.null(l)) r else l
 
 is.named <- function(x) {
