@@ -36,6 +36,7 @@ test_that("if .Renviron overrides R_PROFILE", {
 
   ## But we still need to use the proper lib path, as set in the fake
   ## profile
+  skip_in_covr()
 
   cat("Sys.setenv(FOO='nope')\n", file = tmp_prof <- tempfile())
   cat("R_PROFILE=\"", tmp_prof, "\"\n", file = tmp_env <- tempfile(), sep = "")
@@ -62,11 +63,15 @@ test_that("if .Renviron overrides R_PROFILE", {
 
 test_that("libpath in system(), empty .Renviron", {
 
+  # We remove the library with covr from the lib path, so this
+  # cannot work in a subprocess.
+  skip_in_covr()
+
   dir.create(tmpdrop <- tempfile("drop"))
   dir.create(tmpkeep <- tempfile("keep"))
   on.exit(unlink(c(tmpdrop, tmpkeep), recursive = TRUE), add  = TRUE)
 
-  withr::local_tempfile("tmpenv")
+  tmpenv <- withr::local_tempfile()
   cat("", file = tmpenv)
   withr::local_envvar(c(R_ENVIRON_USER = tmpenv))
 
@@ -80,11 +85,15 @@ test_that("libpath in system(), empty .Renviron", {
 
 test_that("libpath in system, R_LIBS in .Renviron", {
 
+  # We remove the library with covr from the lib path, so this
+  # cannot work in a subprocess.
+  skip_in_covr()
+
   dir.create(tmpdrop <- tempfile("drop"))
   dir.create(tmpkeep <- tempfile("keep"))
   on.exit(unlink(c(tmpdrop, tmpkeep), recursive = TRUE), add  = TRUE)
 
-  withr::local_tempfile("tmpenv")
+  tmpenv <- withr::local_tempfile()
   cat("R_LIBS=\"", tmpdrop, "\"\n", sep = "", file = tmpenv)
   withr::local_envvar(c(R_ENVIRON_USER = tmpenv))
 
@@ -98,11 +107,16 @@ test_that("libpath in system, R_LIBS in .Renviron", {
 })
 
 test_that("libpath in system, R_LIBS", {
+
+  # We remove the library with covr from the lib path, so this
+  # cannot work in a subprocess.
+  skip_in_covr()
+
   dir.create(tmpdrop <- tempfile("drop"))
   dir.create(tmpkeep <- tempfile("keep"))
   on.exit(unlink(c(tmpdrop, tmpkeep), recursive = TRUE), add  = TRUE)
 
-  withr::local_tempfile("tmpenv")
+  tmpenv <- withr::local_tempfile()
   cat("", file = tmpenv)
   withr::local_envvar(c(R_ENVIRON_USER = tmpenv, R_LIBS=tmpdrop))
 
@@ -116,11 +130,16 @@ test_that("libpath in system, R_LIBS", {
 })
 
 test_that("libpath in system, R_LIBS and .Renviron", {
+
+  # We remove the library with covr from the lib path, so this
+  # cannot work in a subprocess.
+  skip_in_covr()
+
   dir.create(tmpdrop <- tempfile("drop"))
   dir.create(tmpkeep <- tempfile("keep"))
   on.exit(unlink(c(tmpdrop, tmpkeep), recursive = TRUE), add  = TRUE)
 
-  withr::local_tempfile("tmpenv")
+  tmpenv <-  withr::local_tempfile()
   cat("R_LIBS=\"", tmpdrop, "\"\n", sep = "", file = tmpenv)
   withr::local_envvar(c(R_ENVIRON_USER = tmpenv, R_LIBS=tmpdrop))
 
@@ -186,29 +205,3 @@ test_that("libpath in system, if subprocess changes R_LIBS #2", {
   ## Close FDs
   gc()
 })
-
-test_that("libpath in system, in R CMD INSTALL", {
-  skip_on_cran()
-
-  csomag <- test_path("fixtures","csomag")
-  tmplib <- tempfile()
-  dir.create(tmplib)
-  on.exit(unlink(tmplib, recursive = TRUE), add = TRUE)
-  dump <- tempfile(fileext = ".rds")
-  on.exit(unlink(dump), add = TRUE)
-
-  out <- callr::r(function(pkg, lib, savefile) {
-    Sys.setenv(CALLR_DUMP_HERE = savefile)
-    ## We need to do this, otherwise install.packages() only keeps the
-    ## first library
-    Sys.unsetenv("_R_CHECK_INSTALL_DEPENDS_")
-    install.packages(pkg, lib = lib, repos = NULL, type = "source")
-  }, list(csomag, tmplib, dump))
-
-  data <- readRDS(dump)
-  ## We test the basename, in case a normalizePath makes dirnames differ
-  expect_true(basename(tmplib) %in% basename(data$libpaths))
-
-  ## Close FDs
-  gc()
-})
